from .engine import Engine, get_current_engine
from .server import run_server
from .manager import Manager
from .command import Command
from gevent import monkey
from . import logger

monkey.patch_all(thread=False, time=False)
engine = Engine()



from .engine import get_current_engine
from datetime import datetime
from pathlib import Path
from . import setting
import logging

logger = None
log_list = []
_is_client = False


def _get_logger():
    global logger
    if not logger:
        handlers = [logging.StreamHandler()]
        if setting.log_path:
            if not Path(setting.log_path).exists():
                raise ValueError('setting.log_path not exist.')
            handlers.append(
                logging.FileHandler(f'{setting.log_path}/{datetime.now().strftime("%Y-%m-%d %H-%M-%S")}.log'))
        logging.basicConfig(level=logging.CRITICAL, handlers=handlers, format='%(message)s')
        logger = logging.getLogger()
    return logger


def _get_engine_name():
    try:
        return get_current_engine().name
    except RuntimeError:
        return ''


def _logging(msg):
    _get_logger().critical(msg)


def _log(level, msg, name):
    if level in setting.log_filter:
        msg = f'[{datetime.now()}] {name if name else _get_engine_name()}({level.upper()}): {msg}'
        _logging(msg)
        if _is_client: log_list.append(msg)


def info(msg, name=None):
    _log('info', msg, name)


def debug(msg, name=None):
    _log('debug', msg, name)


def result(msg, name=None):
    _log('result', msg, name)


def warning(msg, name=None):
    _log('warning', msg, name)


def error(msg, name=None):
    _log('error', msg, name)


def system(msg, name=None):
    _log('system', msg, name)



from .controller import Controller
from toolset import get_localhost
from concurrent import futures
from . import service_pb2_grpc
from pathlib import Path
from . import setting
from . import logger
import importlib
import pickle
import grpc
import time


def run_server(load_file=None, dump_path='.', log_path=None, host='localhost', port=6991):
    """
    启动服务
    :param load_file: 加载任务文件
    :param dump_path: 保存任务文件路径
    :param log_path: 日志保存路径
    :param host: 主机
    :param port: 端口
    """
    if not Path(dump_path).exists(): raise FileExistsError(dump_path)
    if setting.log_path and log_path:
        importlib.reload(logger)
        setting.log_path = log_path
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=4))
    controller = Controller(dump_path)
    if load_file:
        if not Path(load_file).exists(): raise FileExistsError(load_file)
        with open(load_file, 'rb') as f:
            controller.task_set = pickle.load(f)
            controller.task_list = list(controller.task_set.keys())
            controller.task_index = max(controller.task_list) + 1

    service_pb2_grpc.add_ControllerServicer_to_server(controller, server)
    if not server.add_insecure_port(f'{host}:{port}'):
        raise RuntimeError('Failed to add port to server.')
    logger.system(f'Start server --- {get_localhost()}:{port}', 'Server')
    server.start()

    while not controller.is_quit:
        time.sleep(15)
    server.stop(0)
    logger.system('Stop server', 'Server')



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='service.proto',
  package='eloope',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\rservice.proto\x12\x06\x65loope\"]\n\x11SendStatusRequest\x12\r\n\x05tasks\x18\x01 \x01(\x0c\x12\x17\n\x0f\x63ompleted_tasks\x18\x02 \x01(\x0c\x12\x12\n\nfree_count\x18\x03 \x01(\x05\x12\x0c\n\x04logs\x18\x04 \x01(\x0c\"4\n\x12SendStatusResponse\x12\x0f\n\x07\x63ommand\x18\x01 \x01(\t\x12\r\n\x05tasks\x18\x02 \x01(\x0c\"%\n\x12SendCommandRequest\x12\x0f\n\x07\x63ommand\x18\x01 \x01(\t\"#\n\x13SendCommandResponse\x12\x0c\n\x04info\x18\x01 \x01(\t2\x9d\x01\n\nController\x12\x45\n\nSendStatus\x12\x19.eloope.SendStatusRequest\x1a\x1a.eloope.SendStatusResponse\"\x00\x12H\n\x0bSendCommand\x12\x1a.eloope.SendCommandRequest\x1a\x1b.eloope.SendCommandResponse\"\x00\x62\x06proto3')
)




_SENDSTATUSREQUEST = _descriptor.Descriptor(
  name='SendStatusRequest',
  full_name='eloope.SendStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tasks', full_name='eloope.SendStatusRequest.tasks', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='completed_tasks', full_name='eloope.SendStatusRequest.completed_tasks', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='free_count', full_name='eloope.SendStatusRequest.free_count', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='logs', full_name='eloope.SendStatusRequest.logs', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=118,
)


_SENDSTATUSRESPONSE = _descriptor.Descriptor(
  name='SendStatusResponse',
  full_name='eloope.SendStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='eloope.SendStatusResponse.command', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tasks', full_name='eloope.SendStatusResponse.tasks', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=172,
)


_SENDCOMMANDREQUEST = _descriptor.Descriptor(
  name='SendCommandRequest',
  full_name='eloope.SendCommandRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='eloope.SendCommandRequest.command', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=174,
  serialized_end=211,
)


_SENDCOMMANDRESPONSE = _descriptor.Descriptor(
  name='SendCommandResponse',
  full_name='eloope.SendCommandResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='info', full_name='eloope.SendCommandResponse.info', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=213,
  serialized_end=248,
)

DESCRIPTOR.message_types_by_name['SendStatusRequest'] = _SENDSTATUSREQUEST
DESCRIPTOR.message_types_by_name['SendStatusResponse'] = _SENDSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['SendCommandRequest'] = _SENDCOMMANDREQUEST
DESCRIPTOR.message_types_by_name['SendCommandResponse'] = _SENDCOMMANDRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SendStatusRequest = _reflection.GeneratedProtocolMessageType('SendStatusRequest', (_message.Message,), dict(
  DESCRIPTOR = _SENDSTATUSREQUEST,
  __module__ = 'service_pb2'
  # @@protoc_insertion_point(class_scope:eloope.SendStatusRequest)
  ))
_sym_db.RegisterMessage(SendStatusRequest)

SendStatusResponse = _reflection.GeneratedProtocolMessageType('SendStatusResponse', (_message.Message,), dict(
  DESCRIPTOR = _SENDSTATUSRESPONSE,
  __module__ = 'service_pb2'
  # @@protoc_insertion_point(class_scope:eloope.SendStatusResponse)
  ))
_sym_db.RegisterMessage(SendStatusResponse)

SendCommandRequest = _reflection.GeneratedProtocolMessageType('SendCommandRequest', (_message.Message,), dict(
  DESCRIPTOR = _SENDCOMMANDREQUEST,
  __module__ = 'service_pb2'
  # @@protoc_insertion_point(class_scope:eloope.SendCommandRequest)
  ))
_sym_db.RegisterMessage(SendCommandRequest)

SendCommandResponse = _reflection.GeneratedProtocolMessageType('SendCommandResponse', (_message.Message,), dict(
  DESCRIPTOR = _SENDCOMMANDRESPONSE,
  __module__ = 'service_pb2'
  # @@protoc_insertion_point(class_scope:eloope.SendCommandResponse)
  ))
_sym_db.RegisterMessage(SendCommandResponse)



_CONTROLLER = _descriptor.ServiceDescriptor(
  name='Controller',
  full_name='eloope.Controller',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=251,
  serialized_end=408,
  methods=[
  _descriptor.MethodDescriptor(
    name='SendStatus',
    full_name='eloope.Controller.SendStatus',
    index=0,
    containing_service=None,
    input_type=_SENDSTATUSREQUEST,
    output_type=_SENDSTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SendCommand',
    full_name='eloope.Controller.SendCommand',
    index=1,
    containing_service=None,
    input_type=_SENDCOMMANDREQUEST,
    output_type=_SENDCOMMANDRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_CONTROLLER)

DESCRIPTOR.services_by_name['Controller'] = _CONTROLLER

# @@protoc_insertion_point(module_scope)



from gevent.pool import Pool

_current_engine = []


class Engine:
    def __init__(self, name='engine', size=50):
        """
        :param name: 名称
        :param size: 协程池(任务池)大小
        """
        self._name = name
        self._size = size
        self._task_pool = ()
        self._is_run = False
        self._tasks = []

    @property
    def name(self):
        return self._name

    @property
    def is_run(self):
        """运行状态"""
        return self._is_run

    @property
    def task_count(self):
        """任务数量"""
        return len(self._task_pool if self._is_run else self._tasks)

    @property
    def free_count(self):
        """任务池剩余空间"""
        return self._size - self.task_count

    @property
    def task_pool(self):
        """任务池"""
        return (g for g in self._task_pool)

    def run(self):
        from . import logger
        assert not self._is_run, 'Engine object is already running.'
        self._task_pool = Pool(self._size)
        self._is_run = True
        _current_engine.append(self)
        logger.system('Begin')
        for task in self._tasks:
            self._task_pool.spawn(*task)
        self._task_pool.join()
        logger.system('End')
        _current_engine.pop()
        self._task_pool = ()
        self._is_run = False
        self._tasks = []

    def add_task(self, fn, *params):
        """
        添加单个任务
        :param fn: 函数
        :param params: 函数参数
        """
        self._task_pool.spawn(fn, *params) if self._is_run else self._tasks.append((fn, *params))

    def add_tasks(self, fn, param_groups):
        """
        添加多个任务, 同一个函数不同参数
        :param fn: 函数
        :param param_groups: 参数列表
        """
        for param in param_groups:
            self.add_task(fn, *param)


def get_current_engine():
    """获取当前运行的engine"""
    try: return _current_engine[-1]
    except IndexError: raise RuntimeError('No engine is running.')



from .command import Command
import sys


def main():
    cmd = sys.argv[1]
    params = sys.argv[2:]
    host = 'localhost'
    port = 6991
    for i, param in enumerate(params):
        if param in ['-h', '-p']:
            if param == '-h': host = params[i + 1]
            else: port = int(params[i + 1])
            del params[i + 1]
    return Command.execute(cmd, *params, host=host, port=port)



from . import service_pb2_grpc, service_pb2
from datetime import datetime
from pathlib import Path
from . import logger
import pickle


class Controller(service_pb2_grpc.ControllerServicer):
    def __init__(self, dump_path):
        self.start_time = datetime.now()
        self.is_active = True
        self.is_quit = False
        self.dump_path = dump_path
        self.task_index = 0
        self.task_set = {}
        self.task_list = []

    def SendStatus(self, request, context):
        if self.is_quit: return service_pb2.SendStatusResponse(command='quit', tasks=pickle.dumps([]))
        # logging
        for msg in pickle.loads(request.logs):
            logger._logging(msg)
        # Clear tasks
        for code in pickle.loads(request.completed_tasks):
            del self.task_set[code]
        # Append tasks
        for task in pickle.loads(request.tasks):
            self.task_set[self.task_index] = {'fn': task}
            self.task_list.append(self.task_index)
            self.task_index += 1
        logger.system(f'Status - task_count= {len(self.task_list)} task_total= {self.task_index + 1}', 'Server')
        # Return tasks
        if self.is_active and request.free_count > 0:
            task_codes = self.task_list[:request.free_count]
            tasks = []
            for code in task_codes:
                tasks.append((code, self.task_set[code]['fn']))
            self.task_list = self.task_list[request.free_count:]
            return service_pb2.SendStatusResponse(command='add', tasks=pickle.dumps(tasks))
        else: return service_pb2.SendStatusResponse(command='pass', tasks=pickle.dumps([]))

    def SendCommand(self, request, context):
        logger.system(f'Receive command - {request.command}', 'Server')
        if request.command == 'start':
            self.is_active = True
            return service_pb2.SendCommandResponse(info='Start successfully.')
        elif request.command == 'stop':
            self.is_active = False
            return service_pb2.SendCommandResponse(info='Stop successfully.')
        elif request.command == 'save':
            dump_file = Path(self.dump_path) / self.start_time.strftime('%Y-%m-%d %H-%M-%S')
            with open(dump_file, 'wb') as f:
                pickle.dump(self.task_set, f)
            return service_pb2.SendCommandResponse(info=f'Save successfully! file path: {dump_file}.')
        elif request.command == 'quit':
            self.is_active = False
            self.is_quit = True
            return service_pb2.SendCommandResponse(info='Quit successfully! please wait a few seconds...')
        elif request.command == 'task_total':
            return service_pb2.SendCommandResponse(info=str(self.task_index + 1))
        elif request.command == 'task_count':
            return service_pb2.SendCommandResponse(info=str(len(self.task_list)))
        elif request.command == 'start_time':
            return service_pb2.SendCommandResponse(info=str(self.start_time))



from . import service_pb2_grpc, service_pb2
from .server import run_server
import grpc


class Command:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    @classmethod
    def execute(cls, cmd, *args, host='localhost', port=6991):
        return getattr(cls(host, port), cmd, Command.help)(*args)

    def _send_command(self, cmd):
        channel = grpc.insecure_channel(f'{self.host}:{self.port}')
        stub = service_pb2_grpc.ControllerStub(channel)
        resp = stub.SendCommand(service_pb2.SendCommandRequest(command=cmd))
        return resp.info

    @staticmethod
    def help():
        return """View the list of commands (or check the correctness of the command input):
            help --- view the list of commands.
            stop --- stop sending tasks to the client.
            start --- start sending tasks to the client.
            save --- save the task to a file.
            quit --- quit the server and clients.
            task_total --- total number of tasks.
            task_count --- number of tasks on the server.
            start_time --- start time.
            connect --- connect to the server.
            run_server <load_file> <dump_path> <log_path> --- run server.
        """

    def stop(self, *args):
        return self._send_command('stop')

    def start(self, *args):
        return self._send_command('start')

    def save(self, *args):
        return self._send_command('save')

    def quit(self, *args):
        return self._send_command('quit')

    def task_total(self, *args):
        return self._send_command('task_total')

    def task_count(self, *args):
        return self._send_command('task_count')

    def start_time(self, *args):
        return self._send_command('start_time')

    def run_server(self, *args):
        if len(args) > 3:
            return Command.help()
        run_server(*args, host=self.host, port=self.port)
        return 'Quit successfully!'

    def connect(self, *args):
        while True:
            cmd = input('Command: ')
            if cmd == 'exit': break
            if not cmd: continue
            cmd, *args = cmd.strip().split()
            print('-->', getattr(self, cmd, Command.help)(*args))



from . import logger


def _connection_interrupt_callback(engine, repeat_count):
    """
    Engine 连接中断回调函数,
    :param engine: Engine 实例
    :param repeat_count: 重复连接次数
    :return: True: 重新连接, False: 断开连接退出程序
    """
    logger.system(f'Connection interrupt (repeat_count: {repeat_count}) ')
    return True if repeat_count < 5 else False


connection_interrupt_callback = _connection_interrupt_callback
send_interval = 1
log_path = None
log_filter = ('info', 'debug', 'result', 'warning', 'error', 'system')



from . import service_pb2, service_pb2_grpc
from multiprocessing import Process
from types import MethodType
from .engine import Engine
from . import setting
from . import logger
import pickle
import gevent
import grpc
import time


class Manager:
    def __init__(self, engine_names, host='localhost', port=6991):
        self._tasks = []
        self._engines = []
        self._is_run = False
        self._host = host
        self._port = port
        self.create_engines(engine_names)

    def create_engines(self, engine_names, size=50):
        assert not self._is_run, 'Manager object is already running.'
        assert engine_names, 'engine_names is empty.'
        for name in engine_names:
            engine = Engine(name, size)
            engine.add_task(_send_status, engine)
            self._engines.append(engine)

    def add_task(self, fn, *params):
        assert not self._is_run, 'Manager object is already running.'
        self._tasks.append((fn, *params))

    def add_tasks(self, fn, param_groups):
        for param in param_groups:
            self.add_task(fn, *param if isinstance(param, (tuple, list)) else param)

    def run(self):
        assert not self._is_run, 'Manager object is already running.'
        assert self._engines, 'No Engine object added.'
        channel = grpc.insecure_channel(f'{self._host}:{self._port}')
        stub = service_pb2_grpc.ControllerStub(channel)
        # Push initial tasks
        try:
            tasks = pickle.dumps([pickle.dumps(t) for t in self._tasks])
            stub.SendStatus(service_pb2.SendStatusRequest(tasks=tasks, free_count=0, completed_tasks=pickle.dumps([]),
                                                          logs=pickle.dumps([])))
            self._tasks = []
        except grpc.RpcError:
            raise ConnectionError('Connect Failed.')
        # Create process
        for engine in self._engines:
            Process(target=_create_process, args=(engine, self._host, self._port)).start()


def _add_task(self, fn, *params, is_put=True):
    self.task_queue.append((fn, *params)) if is_put else self._task_pool.spawn(fn, *params)


def _create_process(engine, host, port):
    channel = grpc.insecure_channel(f'{host}:{port}')
    logger._is_client = True
    engine.stub = service_pb2_grpc.ControllerStub(channel)
    engine.task_queue = []
    engine.completed_tasks = []
    engine.add_task = MethodType(_add_task, engine)
    logger.system('Create process', engine.name)
    engine.run()


def _package_task(engine, code, fn, *params):
    fn(*params)
    engine.completed_tasks.append(code)


def _send_status(engine):
    repeat_count = 0
    while True:
        # Upload tasks
        try:
            tasks = pickle.dumps([pickle.dumps(t) for t in engine.task_queue])
            logs = pickle.dumps(logger.log_list)
            resp = engine.stub.SendStatus(service_pb2.SendStatusRequest(
                tasks=tasks, free_count=engine.free_count, logs=logs,
                completed_tasks=pickle.dumps(engine.completed_tasks)))
            engine.task_queue = []
            logger.log_list = []
            engine.completed_tasks = []
            if resp.command == 'add':
                # Download tasks
                for code, task in pickle.loads(resp.tasks):
                    engine.add_task(_package_task, engine, code, *pickle.loads(task), is_put=False)
            elif resp.command == 'quit':
                gevent.killall(engine.task_pool)
                break
        except grpc.RpcError:
            # Connection interrupt
            if engine.task_count == 1:
                if setting.connection_interrupt_callback(engine, repeat_count): repeat_count += 1
                else: break
        # Request_interval
        time.sleep(setting.send_interval) if engine.task_count == 1 else gevent.sleep(setting.send_interval)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import service_pb2 as service__pb2


class ControllerStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.SendStatus = channel.unary_unary(
        '/eloope.Controller/SendStatus',
        request_serializer=service__pb2.SendStatusRequest.SerializeToString,
        response_deserializer=service__pb2.SendStatusResponse.FromString,
        )
    self.SendCommand = channel.unary_unary(
        '/eloope.Controller/SendCommand',
        request_serializer=service__pb2.SendCommandRequest.SerializeToString,
        response_deserializer=service__pb2.SendCommandResponse.FromString,
        )


class ControllerServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def SendStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SendCommand(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ControllerServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'SendStatus': grpc.unary_unary_rpc_method_handler(
          servicer.SendStatus,
          request_deserializer=service__pb2.SendStatusRequest.FromString,
          response_serializer=service__pb2.SendStatusResponse.SerializeToString,
      ),
      'SendCommand': grpc.unary_unary_rpc_method_handler(
          servicer.SendCommand,
          request_deserializer=service__pb2.SendCommandRequest.FromString,
          response_serializer=service__pb2.SendCommandResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'eloope.Controller', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
